from setuptools import setup, find_packages
import os

version = '1.0.6'

setup(name='Products.csvreplicata',
      version=version,
      description="CSV import/export for Archetypes",
      long_description='\n'.join(
        open(os.path.join(*path)).read() for path in [
            ("README.txt",),
            ("docs", "HISTORY.txt"),
            ("Products", "csvreplicata", "tests", "csvreplicata.txt")]),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='archetypes, import/export, csv, replication, synchronisation',
      author='Eric BREHAULT',
      author_email='eric.brehault@makina-corpus.org',
      url='http://plone.org/products/csvreplicata',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
